@@ -28,7 +28,6 @@
 extern Client_Charset utf8;
 char LogFile[256];
 
-
 char *strndup(const char *s, size_t n)
 {
   size_t len= MIN(strlen(s), n);
@@ -274,3 +273,26 @@ SQLLEN MADB_SetString(Client_Charset* cc, void *Dest, SQLULEN DestLength,
     return Length;
   }
 }
+
+
+int GetSourceAnsiCs(Client_Charset *cc)
+{
+  cc->CodePage= GetACP();
+
+  /* We don't need cs_info for this */
+  return cc->CodePage;
+}
+
+
+/* {{{ MADB_DSN_SetDefaults() */
+BOOL MADB_DSN_PossibleConnect(MADB_Dsn *Dsn)
+{
+  return Dsn->ServerName && (Dsn->IsNamedPipe || Dsn->Port > 0);
+}
+
+
+char* strcasestr(const char* HayStack, const char* Needle)
+{
+  return StrStrIA(HayStack, Needle);
+}
+/* }}} */
\ No newline at end of file
